#!/usr/bin/python
# -*- coding: utf-8 -*-

##
# NX Client for Windows installer with RegistryHandler and DecideArchitecture classes
# Checks whether NX Client for Windows is installed in the system, the classes used for this
# process are used for other operations too
##

import os, sys, argparse, errno, subprocess, time
if sys.hexversion > 0x02070000:
    from collections import *
else:
    from OrderedDict import *
from _winreg import *

##
# Argument parser, based on argparse module
# @return args
#
def pars_arguments():
    parser = argparse.ArgumentParser();
    if DecideArchitecture.Is64Windows():
        local_default = DecideArchitecture.GetProgramFiles64()+"\\CIRCLE\\"
    else:
        local_default = DecideArchitecture.GetProgramFiles32()+"\\CIRCLE\\"
    if not os.path.exists(local_default[:-1]) and os.path.exists(os.environ['APPDATA']+"\\CIRCLE"):
        local_default = os.environ['APPDATA']+"\\CIRCLE\\"
    parser.add_argument("-l", "--location", help="Location of the client files in the system", default=local_default)
    parser.add_argument("-g", "--global", help="Whether we want to edit registry globally or just locally", action="store_true")
    parser.add_argument("-d", "--different", help="Only handle the architecture specific registry area", action="store_true")
    parser.add_argument("-r", "--registry", help="Which HKEY_* const registry type the program should use", \
        type=str, choices=['HKLM', 'HKCR', 'HKCU', 'HKU', 'HKPD', 'HKCC'], default="HKLM")
    args = parser.parse_args();
    return args

##
# Parameter bypassing struct
#
class Struct:
    pass
    
##
# Registry handling class, makes registry based queries and manipulations easier
#    This class can handle WOW64 based application differently and none differently (default)
# 
class RegistryHandler:
    ##
    # Initialise RegistryHandler 
    # @param args   The args.registry tells us what registry we need to handle
    #  
    def __init__(self, args = None):
        if args is None:
            self.args = Struct()
            self.args.different = None
            self.args.registry = None
        else:
            if not hasattr(args, 'different'):
                args.different = None
            if not hasattr(args, 'registry'):
                args.registry = None
            self.args = args
        if self.args.different is None:
            self.args.different = False
        if self.args.registry is None or self.args.registry == "HKLM":
            self.args.registry = HKEY_LOCAL_MACHINE
        elif self.args.registry == "HKCR":
            self.args.registry = HKEY_CLASSES_ROOT
        elif self.args.registry == "HKCU":
            self.args.registry = HKEY_CURRENT_USER
        elif self.args.registry == "HKU":
            self.args.registry = HKEY_USERS
        elif self.args.registry == "HKPD":
            self.args.registry = HKEY_PERFORMANCE_DATA
        elif self.args.registry == "HKCC":
            self.args.registry = HKEY_CURRENT_CONFIG
        else:
            #print "Non supported registry type"
            raise AttributeError
    ##
    # Getting a registry open
    # @return connected_registy	The registry which we successfully opened
    #    
    def connect_registry(self):
        return ConnectRegistry(None,self.args.registry)
    
    ##
    # Create registry key and value multilevel tree by simple chained dictionaries 
    #   Can raise AttributeError if the provided dict chain isn't correct
    # @param key_value_chain    The dict chain containing all the information
    # @param both               Whether create the registry in WOW64 node too
    # @param architect          The current registry view (only change it if we want to fill the
    #                           WOW6432 registry only [both = False], on x64 windows)
    # @param needed_rights      SAM rights to access the key
    #
    def create_registry_from_dict_chain(self, dict_chain, both = False, architect = KEY_WOW64_64KEY, needed_rights = KEY_ALL_ACCESS):
        if both and architect is KEY_WOW64_64KEY:
            self.create_registry_from_dict_chain(dict_chain, False, KEY_WOW64_32KEY, needed_rights)
        if not DecideArchitecture.Is64Windows():
            architect = 0
        connected_registy = self.connect_registry()
        if isinstance(dict_chain, dict) or isinstance(dict_chain, OrderedDict):
            for key, value in dict_chain.iteritems():
                if isinstance(value, dict) or isinstance(value, OrderedDict):
                    temp_dict = OrderedDict()
                    for my_key, my_value in value.iteritems():
                        temp_dict[key+"\\"+my_key] = my_value             
                    self.create_registry_from_dict_chain(temp_dict, False, architect, needed_rights)
                else:
                    if isinstance(value, list):
                        if len(value)%2 != 0:
                            #print "Not enough member in the list"
                            raise AttributeError
                        else:
                            new_key = CreateKeyEx(connected_registy, key, 0, needed_rights | architect)
                            temp_dict = OrderedDict(value[i:i+2] for i in range(0, len(value), 2))
                            for my_key, my_value in temp_dict.iteritems():
                                SetValueEx(new_key,my_key,0, REG_SZ, my_value)
                    else:        
                        new_key = CreateKeyEx(connected_registy, key, 0, needed_rights | architect)
                        SetValueEx(new_key,None,0, REG_SZ, value)
        else:
            print "The provided attribute wasn't a dictionary chain"
            raise AttributeError
    ##
    # Getting a registry value by it's key's name
    #   Can raise KeyError if key is not found in the registry
    # @param key_name       The specific key name of which value we are interested in
    # @param needed_rights  SAM rights to access the key
    # @return [key, architect]
    #   key:        The reference to the opened key handler
    #   architect:  0 for x86
    #               KEY_WOW64_32KEY or KEY_WOW64_64KEY depending where we found the key on x64
    #
    def get_key(self, key_name, needed_rights = KEY_ALL_ACCESS):
        connected_registy = self.connect_registry()
        architect = 0
        if DecideArchitecture.Is64Windows():
            architect = KEY_WOW64_64KEY    
        try: 
            key = OpenKey(connected_registy, key_name, 0, needed_rights | architect)
        except WindowsError:
            if DecideArchitecture.Is64Windows() and not self.args.different:
                try:
                    architect = KEY_WOW64_32KEY
                    key = OpenKey(connected_registy, key_name, 0, needed_rights | architect)
                except WindowsError:
                    raise KeyError
            else:
                raise KeyError
        return [key, architect]
        
    ##
    # Getting registry subkeys value by it's key's name and subkeys name
    #   Can raise LookupError exception if there are missing data
    #   Can raise AttributeError exception if depth attribute is wrong
    # @param key_name       The specific key name of which subkey's we are interested in
    # @param subkey_list    List containing all the subkeys names which values we are interested in
    # @param subroutine     Whether suppress exception about not having enough results or not - default False
    # @param depth          How depth the search should go for (key, subkeys, all) - default: subkeys 
    # @return results{}     Dictionary with the subkey_name - value combinations as keys and values
    #
    def get_key_values(self, key_name, subkey_list, subroutine = False, depth = "subkeys"):
        if depth == "key":
            int_depth = 0;
        elif depth == "subkeys":
            int_depth = 1;
        elif depth == "all":
            int_depth = 2;
        else:
            raise AttributeError
        try:
            key_and_architect = self.get_key(key_name)
            key = key_and_architect[0]
            architect = key_and_architect[1]
        except KeyError:
            #print key_name+" doesn't exist in the registry"
            raise LookupError
        #print key_name+" found in the registry"
        results = {}
        if int_depth >= 1:
            for i in xrange(0, QueryInfoKey(key)[0]-1):
                skey_name = EnumKey(key, i)
                skey = OpenKey(key, skey_name, 0, KEY_ALL_ACCESS | architect)
                if int_depth == 2 and QueryInfoKey(skey)[0] > 0:
                    for key, value in self.get_key_values(skey_name, subkey_list, True, depth).iteritems():
                        results[key] = value
                for subkey_name in subkey_list:
                    try:
                        results[subkey_name] = QueryValueEx(skey, subkey_name)[0]
                    except OSError as e:
                        if e.errno == errno.ENOENT:
                            #print subkey_name+" doesn't exist in this subkey"
                            pass
                skey.Close()
        if not results or len(results) != len(subkey_list):
             for subkey_name in subkey_list:
                try:
                    results[subkey_name] = QueryValueEx(key, subkey_name)[0]
                except OSError as e:
                    pass
        key.Close()
        if len(results) != len(subkey_list):
            #print We are missing important variables
            raise LookupError
        return results
    
    ##
    # This is a wrapper for the get_key_values to be easier to use for single subkeys
    # Getting registry subkey value by it's key's name and subkey name
    # @param key_name    The specific key name of which subkey's we are interested in
    # @param subkey_name The specific subkey name which value we are interested in
    # @param subroutine  can be found at: get_key_values
    # @param depth       can be found at: get_key_values
    # @return value      Value of the specific subkey
    #
    def get_key_value(self, key_name, subkey_name, subroutine = None, depth = None):
        try:
            if subroutine is None:
                return self.get_key_values(key_name, [subkey_name])[subkey_name]
            elif depth is None:
                return self.get_key_values(key_name, [subkey_name], subroutine)[subkey_name]
            else:
                return self.get_key_values(key_name, [subkey_name], subroutine, depth)[subkey_name]
        except:
            raise
##
# Helper class to get the true ProgramFiles directory.
#    This class doesn't depend on Phyton or Windows architecture.
#          
class DecideArchitecture:
    @staticmethod
    def Is64Windows():
        return 'PROGRAMFILES(X86)' in os.environ
    @staticmethod
    def GetProgramFiles32():
        if DecideArchitecture.Is64Windows():
            return os.environ['PROGRAMFILES(X86)']
        else:
            return os.environ['PROGRAMFILES']
    @staticmethod
    def GetProgramFiles64():
        if DecideArchitecture.Is64Windows():
            return os.environ['PROGRAMW6432']
        else:
            return None

def main():
    try:
        args = pars_arguments()
        nx_install_location = None
        while nx_install_location is None:
            print "Checking whether NX Client for Windows is installed"
            handler = RegistryHandler()
            try:
                nx_install_location = handler.get_key_value("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\nxclient_is1", "InstallLocation", "key")
                print "NX Client for Windows is found at '"+nx_install_location+"'"
                process = subprocess.Popen("%s\\nxclient.exe" % nx_install_location)
                time.sleep(2)
                process.terminate()
            except:
                print "NX Client for Windows isn't installed on the system."
                print "\tCommencing the install"
                subprocess.Popen(os.path.dirname(os.path.realpath(__file__))+"\\nxclient-3.5.0-9.exe").wait()
    except:
        pass
if __name__ == "__main__":
    main()